@@ -160,6 +160,14 @@
  */
 #define LIBKECCAK_GENERALISED_SPEC_ERROR_OUTPUT_NONPOSITIVE 11
 
+/**
+ * Invalid `struct libkeccak_generalised_spec.state_size`,
+ * `struct libkeccak_generalised_spec.bitrate`, and
+ * `struct libkeccak_generalised_spec.capacity`:
+ * `.bitrate + .capacity != .state_size`
+ */
+#define LIBKECCAK_GENERALISED_SPEC_ERROR_STATE_BITRATE_CAPACITY_INCONSISTENCY 12
+
 
 /**
  * Data structure that describes the parameters
@@ -289,7 +297,7 @@ struct libkeccak_state {
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__)))
 inline void
-libkeccak_spec_sha3(struct libkeccak_spec *restrict spec, long int x)
+libkeccak_spec_sha3(struct libkeccak_spec *spec, long int x)
 {
 	spec->bitrate = 1600 - 2 * x;
 	spec->capacity = 2 * x;
@@ -305,7 +313,7 @@ libkeccak_spec_sha3(struct libkeccak_spec *restrict spec, long int x)
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__)))
 inline void
-libkeccak_spec_rawshake(struct libkeccak_spec *restrict spec, long int x, long int d)
+libkeccak_spec_rawshake(struct libkeccak_spec *spec, long int x, long int d)
 {
 	spec->bitrate = 1600 - 2 * x;
 	spec->capacity = 2 * x;
@@ -321,6 +329,15 @@ libkeccak_spec_rawshake(struct libkeccak_spec *restrict spec, long int x, long i
  */
 #define libkeccak_spec_shake libkeccak_spec_rawshake
 
+/**
+ * Fill in a `struct libkeccak_spec` for a cSHAKEx hashing
+ * 
+ * @param  spec:struct libkeccak_spec *  The specifications datastructure to fill in
+ * @param  x:long                        The value of x in `cSHAKEx`, half the capacity
+ * @param  d:long                        The output size
+ */
+#define libkeccak_spec_cshake libkeccak_spec_rawshake
+
 /**
  * Check for errors in a `struct libkeccak_spec`
  * 
@@ -329,7 +346,7 @@ libkeccak_spec_rawshake(struct libkeccak_spec *restrict spec, long int x, long i
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__, __warn_unused_result__, __pure__)))
 inline int
-libkeccak_spec_check(const struct libkeccak_spec *restrict spec)
+libkeccak_spec_check(const struct libkeccak_spec *spec)
 {
 	long int state_size = spec->capacity + spec->bitrate;
 	int32_t word_size = (int32_t)(state_size / 25);
@@ -360,7 +377,7 @@ libkeccak_spec_check(const struct libkeccak_spec *restrict spec)
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__)))
 inline void
-libkeccak_generalised_spec_initialise(struct libkeccak_generalised_spec *restrict spec)
+libkeccak_generalised_spec_initialise(struct libkeccak_generalised_spec *spec)
 {
 	spec->bitrate    = LIBKECCAK_GENERALISED_SPEC_AUTOMATIC;
 	spec->capacity   = LIBKECCAK_GENERALISED_SPEC_AUTOMATIC;
@@ -377,7 +394,7 @@ libkeccak_generalised_spec_initialise(struct libkeccak_generalised_spec *restric
  * @return               Zero if `spec` is valid, a `LIBKECCAK_GENERALISED_SPEC_ERROR_*` if an error was found
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__leaf__, __nonnull__, __nothrow__)))
-int libkeccak_degeneralise_spec(struct libkeccak_generalised_spec *restrict, struct libkeccak_spec *restrict);
+int libkeccak_degeneralise_spec(struct libkeccak_generalised_spec *, struct libkeccak_spec *);
 
 /**
  * Initialise a state according to hashing specifications
@@ -387,7 +404,7 @@ int libkeccak_degeneralise_spec(struct libkeccak_generalised_spec *restrict, str
  * @return         Zero on success, -1 on error
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__leaf__, __nonnull__)))
-int libkeccak_state_initialise(struct libkeccak_state *restrict, const struct libkeccak_spec *restrict);
+int libkeccak_state_initialise(struct libkeccak_state *, const struct libkeccak_spec *);
 
 /**
  * Reset a state according to hashing specifications
@@ -396,7 +413,7 @@ int libkeccak_state_initialise(struct libkeccak_state *restrict, const struct li
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__)))
 inline void
-libkeccak_state_reset(struct libkeccak_state *restrict state)
+libkeccak_state_reset(struct libkeccak_state *state)
 {
 	state->mptr = 0;
 	memset(state->S, 0, sizeof(state->S));
@@ -408,7 +425,7 @@ libkeccak_state_reset(struct libkeccak_state *restrict state)
  * @param  state  The state that should be destroyed
  */
 inline void
-libkeccak_state_fast_destroy(struct libkeccak_state *restrict state)
+libkeccak_state_fast_destroy(struct libkeccak_state *state)
 {
 	if (state) {
 		free(state->M);
@@ -422,7 +439,7 @@ libkeccak_state_fast_destroy(struct libkeccak_state *restrict state)
  * @param  state  The state that should be wipe
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__leaf__, __nonnull__, __nothrow__, __optimize__("-O0"))))
-void libkeccak_state_wipe_message(volatile struct libkeccak_state *restrict);
+void libkeccak_state_wipe_message(volatile struct libkeccak_state *);
 
 /**
  * Wipe data in the state's sponge wihout freeing any data
@@ -430,7 +447,7 @@ void libkeccak_state_wipe_message(volatile struct libkeccak_state *restrict);
  * @param  state  The state that should be wipe
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__leaf__, __nonnull__, __nothrow__, __optimize__("-O0"))))
-void libkeccak_state_wipe_sponge(volatile struct libkeccak_state *restrict);
+void libkeccak_state_wipe_sponge(volatile struct libkeccak_state *);
 
 /**
  * Wipe sensitive data wihout freeing any data
@@ -438,7 +455,7 @@ void libkeccak_state_wipe_sponge(volatile struct libkeccak_state *restrict);
  * @param  state  The state that should be wipe
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__, __optimize__("-O0"))))
-void libkeccak_state_wipe(volatile struct libkeccak_state *restrict);
+void libkeccak_state_wipe(volatile struct libkeccak_state *);
 
 /**
  * Release resources allocation for a state and wipe sensitive data
@@ -447,7 +464,7 @@ void libkeccak_state_wipe(volatile struct libkeccak_state *restrict);
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__optimize__("-O0"))))
 inline void
-libkeccak_state_destroy(volatile struct libkeccak_state *restrict state)
+libkeccak_state_destroy(volatile struct libkeccak_state *state)
 {
 	if (state) {
 		libkeccak_state_wipe(state);
@@ -463,7 +480,7 @@ libkeccak_state_destroy(volatile struct libkeccak_state *restrict state)
  * @return        The state, `NULL` on error
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __warn_unused_result__, __malloc__)))
-struct libkeccak_state *libkeccak_state_create(const struct libkeccak_spec *restrict);
+struct libkeccak_state *libkeccak_state_create(const struct libkeccak_spec *);
 
 /**
  * Wrapper for `libkeccak_state_fast_destroy` that also frees the allocation of the state
@@ -471,7 +488,7 @@ struct libkeccak_state *libkeccak_state_create(const struct libkeccak_spec *rest
  * @param  state  The state that should be freed
  */
 inline void
-libkeccak_state_fast_free(struct libkeccak_state *restrict state)
+libkeccak_state_fast_free(struct libkeccak_state *state)
 {
 	libkeccak_state_fast_destroy(state);
 	free(state);
@@ -484,7 +501,7 @@ libkeccak_state_fast_free(struct libkeccak_state *restrict state)
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__optimize__("-O0"))))
 inline void
-libkeccak_state_free(volatile struct libkeccak_state *restrict state)
+libkeccak_state_free(volatile struct libkeccak_state *state)
 {
 #ifdef __GNUC__
 # pragma GCC diagnostic push
@@ -514,7 +531,7 @@ int libkeccak_state_copy(struct libkeccak_state *restrict, const struct libkecca
  * @return       The duplicate, `NULL` on error
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __warn_unused_result__, __malloc__)))
-struct libkeccak_state *libkeccak_state_duplicate(const struct libkeccak_state *restrict);
+struct libkeccak_state *libkeccak_state_duplicate(const struct libkeccak_state *);
 
 /**
  * Marshal a `struct libkeccak_state` into a buffer
@@ -537,6 +554,30 @@ size_t libkeccak_state_marshal(const struct libkeccak_state *restrict, void *res
 LIBKECCAK_GCC_ONLY(__attribute__((__leaf__, __nonnull__(2))))
 size_t libkeccak_state_unmarshal(struct libkeccak_state *restrict, const void *restrict);
 
+/**
+ * Create and absorb the initialisation blocks for cSHAKE hashing
+ * 
+ * @param  state       The hashing state
+ * @param  n_text      Function name-string
+ * @param  n_len       Byte-length of `n_text` (only whole byte)
+ * @param  n_bits      Bit-length of `n_text`, minus `n_len * 8`
+ * @param  n_suffix    Bit-string, represented by a NUL-terminated
+ *                     string of '1':s and '0's:, making up the part
+ *                     after `n_text` of the function-name bit-string;
+ *                     `NULL` is treated as the empty string
+ * @param  s_text      Customisation-string
+ * @param  s_len       Byte-length of `s_text` (only whole byte)
+ * @param  s_bits      Bit-length of `s_text`, minus `s_len * 8`
+ * @param  s_suffix    Bit-string, represented by a NUL-terminated
+ *                     string of '1':s and '0's:, making up the part
+ *                     after `s_text` of the customisation bit-string;
+ *                     `NULL` is treated as the empty string
+ */
+LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__(1), __nothrow__)))
+void libkeccak_cshake_initialise(struct libkeccak_state *restrict,
+                                 const void *, size_t, size_t, const char *,
+                                 const void *, size_t, size_t, const char *);
+
 /**
  * Get the number of bytes that are absorbed during
  * one pass of the absorption phase
@@ -546,9 +587,9 @@ size_t libkeccak_state_unmarshal(struct libkeccak_state *restrict, const void *r
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__, __warn_unused_result__, __pure__)))
 inline size_t
-libkeccak_zerocopy_chunksize(struct libkeccak_state *restrict state)
+libkeccak_zerocopy_chunksize(struct libkeccak_state *state)
 {
-	return state->r >> 3;
+	return (size_t)state->r >> 3;
 }
 
 /**
@@ -593,6 +634,19 @@ int libkeccak_fast_update(struct libkeccak_state *restrict, const void *restrict
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__)))
 int libkeccak_update(struct libkeccak_state *restrict, const void *restrict, size_t);
 
+/**
+ * Get message suffix for cSHAKE hashing
+ * 
+ * @param   nlen  Whether the function-name bit string is non-empty
+ * @param   slen  Whether the customisation bit string is non-empty
+ * @return        The message suffix to use
+ */
+LIBKECCAK_GCC_ONLY(__attribute__((__nothrow__, __warn_unused_result__, __const__, __returns_nonnull__)))
+inline const char *
+libkeccak_cshake_suffix(size_t nlen, size_t slen)
+{
+	return (nlen || slen) ? "00" : LIBKECCAK_SHAKE_SUFFIX;
+}
 
 /**
  * Absorb the last part of the message and squeeze the Keccak sponge
@@ -656,7 +710,7 @@ int libkeccak_digest(struct libkeccak_state *restrict, const void *restrict, siz
  * @param  times  The number of rounds
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__)))
-void libkeccak_simple_squeeze(register struct libkeccak_state *restrict, register long int);
+void libkeccak_simple_squeeze(register struct libkeccak_state *, register long int);
 
 /**
  * Squeeze as much as is needed to get a digest a number of times
@@ -665,7 +719,7 @@ void libkeccak_simple_squeeze(register struct libkeccak_state *restrict, registe
  * @param  times  The number of digests
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__)))
-void libkeccak_fast_squeeze(register struct libkeccak_state *restrict, register long int);
+void libkeccak_fast_squeeze(register struct libkeccak_state *, register long int);
 
 /**
  * Squeeze out another digest
@@ -711,14 +765,17 @@ void libkeccak_unhex(void *restrict, const char *restrict);
  * the content of the file is assumed non-sensitive
  * 
  * @param   fd       The file descriptor of the file to hash
- * @param   state    The hashing state, should not be initialised (memory leak otherwise)
+ * @param   state    The hashing state, should not be initialised unless
+ *                   `spec` is `NULL` (memory leak otherwise)
+ * @param   spec     Specifications for the hashing algorithm; or `NULL`
+ *                   if `spec` is already initialised
  * @param   spec     Specifications for the hashing algorithm
  * @param   suffix   The data suffix, see `libkeccak_digest`
  * @param   hashsum  Output array for the hashsum, have an allocation size of
  *                   at least `((spec->output + 7) / 8) * sizeof(char)`, may be `NULL`
  * @return           Zero on success, -1 on error
  */
-LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__(2, 3))))
+LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__(2))))
 int libkeccak_generalised_sum_fd(int, struct libkeccak_state *restrict, const struct libkeccak_spec *restrict,
                                  const char *restrict, void *restrict);
 
@@ -803,11 +860,14 @@ libkeccak_shakesum_fd(int fd, struct libkeccak_state *restrict state, long semic
 	return libkeccak_generalised_sum_fd(fd, state, &spec, LIBKECCAK_SHAKE_SUFFIX, hashsum);
 }
 
+/* TODO add libkeccak_cshakesum_fd */
+
 
 /*
  * The Keccak hash-function, that was selected by NIST as the SHA-3 competition winner,
  * doesn't need this nested approach and can be used to generate a MAC by simply prepending
- * the key to the message. [http://keccak.noekeon.org]
+ * the key to the message. [http://keccak.noekeon.org] HMAC-SHA3-224, HMAC-SHA3-256,
+ * HMAC-SHA3-384, and HMAC-SHA3-512 are however approved by NIST.
  */
 
 
@@ -853,7 +913,7 @@ struct libkeccak_hmac_state {
 	 */
 	unsigned char leftover;
 
-	char __pad[sizeof(void *) - 1];
+	char _pad[sizeof(void *) - 1];
 };
 
 
@@ -906,7 +966,7 @@ LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __warn_unused_result__, __malloc_
 inline struct libkeccak_hmac_state *
 libkeccak_hmac_create(const struct libkeccak_spec *restrict spec, const void *restrict key, size_t key_length)
 {
-	struct libkeccak_hmac_state *restrict state = malloc(sizeof(struct libkeccak_hmac_state));
+	struct libkeccak_hmac_state *state = malloc(sizeof(struct libkeccak_hmac_state));
 	if (!state || libkeccak_hmac_initialise(state, spec, key, key_length)) {
 		free(state);
 		return NULL;
@@ -937,7 +997,7 @@ libkeccak_hmac_reset(struct libkeccak_hmac_state *restrict state, const void *re
  * @param  state  The state that should be wipe
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __nothrow__, __optimize__("-O0"))))
-void libkeccak_hmac_wipe(volatile struct libkeccak_hmac_state *restrict);
+void libkeccak_hmac_wipe(volatile struct libkeccak_hmac_state *);
 
 /**
  * Release resources allocation for an HMAC hashing-state without wiping sensitive data
@@ -945,7 +1005,7 @@ void libkeccak_hmac_wipe(volatile struct libkeccak_hmac_state *restrict);
  * @param  state  The state that should be destroyed
  */
 inline void
-libkeccak_hmac_fast_destroy(struct libkeccak_hmac_state *restrict state)
+libkeccak_hmac_fast_destroy(struct libkeccak_hmac_state *state)
 {
 	if (!state)
 		return;
@@ -965,7 +1025,7 @@ libkeccak_hmac_fast_destroy(struct libkeccak_hmac_state *restrict state)
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__optimize__("-O0"))))
 inline void
-libkeccak_hmac_destroy(volatile struct libkeccak_hmac_state *restrict state)
+libkeccak_hmac_destroy(volatile struct libkeccak_hmac_state *state)
 {
 	if (!state)
 		return;
@@ -986,7 +1046,7 @@ libkeccak_hmac_destroy(volatile struct libkeccak_hmac_state *restrict state)
  * @param  state  The state that should be freed
  */
 inline void
-libkeccak_hmac_fast_free(struct libkeccak_hmac_state *restrict state)
+libkeccak_hmac_fast_free(struct libkeccak_hmac_state *state)
 {
 	libkeccak_hmac_fast_destroy(state);
 	free(state);
@@ -999,14 +1059,14 @@ libkeccak_hmac_fast_free(struct libkeccak_hmac_state *restrict state)
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__optimize__("-O0"))))
 inline void
-libkeccak_hmac_free(volatile struct libkeccak_hmac_state *restrict state)
+libkeccak_hmac_free(volatile struct libkeccak_hmac_state *state)
 {
 #ifdef __GNUC__
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wcast-qual"
 #endif
 	libkeccak_hmac_destroy(state);
-	free((struct libkeccak_hmac_state*)state);
+	free((struct libkeccak_hmac_state *)state);
 #ifdef __GNUC__
 # pragma GCC diagnostic pop
 #endif
@@ -1030,9 +1090,9 @@ int libkeccak_hmac_copy(struct libkeccak_hmac_state *restrict, const struct libk
  */
 LIBKECCAK_GCC_ONLY(__attribute__((__nonnull__, __warn_unused_result__, __malloc__)))
 inline struct libkeccak_hmac_state *
-libkeccak_hmac_duplicate(const struct libkeccak_hmac_state *restrict src)
+libkeccak_hmac_duplicate(const struct libkeccak_hmac_state *src)
 {
-	struct libkeccak_hmac_state *restrict dest = malloc(sizeof(struct libkeccak_hmac_state));
+	struct libkeccak_hmac_state *dest = malloc(sizeof(struct libkeccak_hmac_state));
 	if (!dest || libkeccak_hmac_copy(dest, src)) {
 		libkeccak_hmac_free(dest);
 		return NULL;
